@@ -35,6 +35,7 @@
 
 #include <string.h>
 #include <stdlib.h>
+#define PL_ARITY_AS_SIZE 1
 #include <SWI-Stream.h>
 #include <SWI-Prolog.h>
 #include <config.h>
@@ -72,11 +73,15 @@ static atom_t ATOM_cursor;
 static atom_t ATOM_redisplay;
 static atom_t ATOM_error;
 static atom_t ATOM_fatal;
+static atom_t ATOM_clear;
+static atom_t ATOM_setsize;
+static atom_t ATOM_setunique;
 
 static functor_t FUNCTOR_line2;
 static functor_t FUNCTOR_electric3;
+static functor_t FUNCTOR_pair2;
 
-#define STR_OPTIONS (CVT_ATOM|CVT_STRING|REP_MB|CVT_EXCEPTION)
+#define STR_OPTIONS (CVT_ATOM|CVT_STRING|CVT_LIST|REP_MB|CVT_EXCEPTION)
 
 
 		 /*******************************
@@ -113,6 +118,7 @@ typedef struct el_context
   IOSTREAM	       *ostream;	/* Output stream */
   IOSTREAM	       *estream;	/* Error stream */
   EditLine	       *el;		/* EditLine context */
+  char *		buffered;	/* Buffered long line */
   int			sig_no;		/* For read_char() */
   HistEvent		ev;		/* History event */
   History	       *history;	/* Complete history */
@@ -121,6 +127,7 @@ typedef struct el_context
   IOFUNCTIONS	        functions;	/* SIO function block */
   command	       *commands;	/* User commands */
   binding	       *bindings;	/* Bindings to user commands */
+  int			reader;		/* Current reader thread */
   struct
   { int timeout;			/* Time to wait */
     int move;				/* Amount to move */
@@ -159,6 +166,22 @@ get_context_from_handle(void *handle)
   return NULL;
 }
 
+static el_context *
+get_context_from_ohandle(void *handle)
+{ el_context *c;
+
+  for(c=el_clist; c; c=c->next)
+  { IOSTREAM *s;
+
+    if ( (s=c->ostream) && s->handle == handle )
+      return c;
+    if ( (s=c->estream) && s->handle == handle )
+      return c;
+  }
+
+  return NULL;
+}
+
 static el_context *
 alloc_context(int fd)
 { el_context *c = PL_malloc(sizeof(*c));
@@ -242,6 +265,7 @@ static sigstate el_signals[] =
   { SIGTERM },
   { SIGQUIT },
   { SIGWINCH },
+  { SIGUSR2 },				/* SWI-Prolog thread alert */
   { -1 }
 };
 
@@ -337,10 +361,16 @@ el_sighandler(int sig)
 static const char *
 el_siggets(EditLine *el, int *count)
 { const char *line;
+  FILE *in;
 
-  prepare_signals(el_signals);
-  line = el_gets(el, count);
-  restore_signals(el_signals);
+  el_get(el, EL_GETFP, 0, &in);
+  if ( fileno(in) == 0 )
+  { prepare_signals(el_signals);
+    line = el_gets(el, count);
+    restore_signals(el_signals);
+  } else
+  { line = el_gets(el, count);
+  }
 
   return line;
 }
@@ -495,6 +525,30 @@ read__fixio(int fd, int e)
   }
 }
 
+static ssize_t
+do_read(el_context *ctx, int fd, char *buf, size_t size)
+{ ssize_t rc;
+  int oreader = ctx->reader;
+
+  ctx->reader = PL_thread_self();
+  rc = read(fd, buf, size);
+  ctx->reader = oreader;
+
+  return rc;
+}
+
+
+static void
+refresh(el_context *ctx)
+{ FILE *err;
+
+  el_get(ctx->el, EL_GETFP, 2, &err);
+  el_resize(ctx->el);
+  fprintf(err, "\r");
+  el_set(ctx->el, EL_REFRESH);
+}
+
+
 #ifdef HAVE_EL_WSET
 #define el_char_t wchar_t
 #else
@@ -551,15 +605,15 @@ read_char(EditLine *el, el_char_t *cp)
     return -1;
   }
 
-  while ((num_read = read(fileno(in), cbuf + cbp, (size_t)1)) == -1) {
-    int e = errno;
+  while ( (num_read = do_read(ctx, fileno(in), cbuf + cbp, (size_t)1)) == -1 )
+  { int e = errno;
 
     switch (ctx->sig_no)
     { case SIGCONT:
 	el_set(el, EL_REFRESH);
         goto again;
       case SIGWINCH:
-	el_resize(el);
+	refresh(ctx);
 	goto again;
       default:
 	break;
@@ -569,6 +623,8 @@ read_char(EditLine *el, el_char_t *cp)
       *cp = (el_char_t)'\0';
       return -1;
     }
+    if ( e == EINTR )
+      continue;
 
     if ( !tried && read__fixio(fileno(in), e) == 0 )
     { errno = save_errno;
@@ -580,6 +636,9 @@ read_char(EditLine *el, el_char_t *cp)
     }
   }
 
+  if ( ctx->sig_no == SIGWINCH )
+    refresh(ctx);
+
   /* Test for EOF */
   if ( num_read == 0 )
   { *cp = (el_char_t)'\0';
@@ -638,12 +697,56 @@ read_char(EditLine *el, el_char_t *cp)
 		 *	    IO FUNCTIONS	*
 		 *******************************/
 
+#define ISUTF8_CB(c) (((c)&0xc0) == 0x80)
+
+/* Find the longest prefix of `in` upto `len` of complete
+   UTF-8 characters.
+*/
+
+static size_t
+utf8_chars(const char *in, size_t len)
+{ const char *e = &in[len];
+
+  while ( e > in && ISUTF8_CB(e[-1]) )
+    e--;
+
+  return e-in;
+}
+
+
+static size_t
+send_one_buffer(el_context *ctx, const char *line, char *buf, size_t size)
+{ size_t linelen = strlen(line);
+
+  if ( linelen <= size )
+  { memcpy(buf, line, linelen);
+    ctx->buffered = NULL;
+    return linelen;
+  } else
+  { size_t slen = utf8_chars(line, size);
+
+    memcpy(buf, line, slen);
+    if ( (ctx->buffered = strdup(&line[slen])) )
+      return slen;
+    return -1;				/* ENOMEM */
+  }
+}
+
+
 static ssize_t
 Sread_libedit(void *handle, char *buf, size_t size)
 { el_context *ctx = get_context_from_handle(handle);
   int ttymode = PL_ttymode(ctx->istream);
   int rval;
 
+  if ( ctx->buffered )
+  { char *old = ctx->buffered;
+    size_t slen = send_one_buffer(ctx, old, buf, size);
+
+    free(old);
+    return slen;
+  }
+
   switch( ttymode )
   { case PL_RAWTTY:			/* get_single_char/1 */
     case PL_NOTTY:			/* -tty */
@@ -657,19 +760,14 @@ Sread_libedit(void *handle, char *buf, size_t size)
     }
     case PL_COOKEDTTY:
     default:
-    { const char *line;
-      int len;
+    { int len;
+      const char *line;
 
+      if ( ctx->ostream )
+	Sflush(ctx->ostream);
       update_prompt(ctx);
       if ( (line = el_siggets(ctx->el, &len)) && len > 0 )
-      { size_t linelen = strlen(line);
-
-	if ( linelen <= size )
-	{ memcpy(buf, line, linelen);
-	  return linelen;
-	} else
-	{ assert(0);				/* TBD: buffer */
-	}
+      { return send_one_buffer(ctx, line, buf, size);
       } else if ( len == 0 )
       { return 0;
       } else
@@ -680,6 +778,25 @@ Sread_libedit(void *handle, char *buf, size_t size)
 }
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+The write handler  is  defined  to   deal  with  writes  from background
+threads.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+static ssize_t
+Swrite_libedit(void *handle, char *buf, size_t size)
+{ el_context *ctx = get_context_from_ohandle(handle);
+
+  if ( ctx->reader &&
+       ctx->reader != PL_thread_self() )
+  { // fprintf(stderr, "background write %p\n", ctx);
+    ctx->sig_no = SIGWINCH;			/* simulate a window change */
+  }
+
+  return (*ctx->orig_functions->write)(handle, buf, size);
+}
+
+
 static char *
 prompt(EditLine *el)
 { el_context *ctx;
@@ -723,7 +840,8 @@ pl_wrap(term_t progid, term_t tin, term_t tout, term_t terr)
       ctx->estream = err;
 
       ctx->history = history_init();
-      history(ctx->history, &ctx->ev, H_SETSIZE, 100);
+      history(ctx->history, &ctx->ev, H_SETSIZE,   100);
+      history(ctx->history, &ctx->ev, H_SETUNIQUE, TRUE);
 
       ctx->el = el_init(prog, fin, fout, ferr);
 
@@ -738,12 +856,13 @@ pl_wrap(term_t progid, term_t tin, term_t tout, term_t terr)
       el_set( ctx->el, EL_CLIENTDATA, ctx);
       electric_init(ctx->el);
 
-      ctx->orig_functions = in->functions;
-      ctx->functions      = *in->functions;
-      ctx->functions.read = Sread_libedit;
+      ctx->orig_functions  = in->functions;
+      ctx->functions       = *in->functions;
+      ctx->functions.read  = Sread_libedit;
+      ctx->functions.write = Swrite_libedit;
 
       in->functions  = &ctx->functions;
-      out->functions = &ctx->functions;		/* Why? */
+      out->functions = &ctx->functions;
       err->functions = &ctx->functions;
 
       rc = TRUE;
@@ -1263,6 +1382,120 @@ pl_read_history(term_t tin, term_t file_name)
 }
 
 
+static int
+append_ev(term_t tail, term_t head, const HistEvent *ev)
+{ return ( PL_unify_list(tail, head, tail) &&
+	   PL_unify_term(head, PL_FUNCTOR, FUNCTOR_pair2,
+			 PL_INT, ev->num,
+			 PL_UTF8_STRING, ev->str) );
+}
+
+
+static foreign_t
+pl_history_events(term_t tin, term_t events)
+{ el_context *ctx;
+
+  if ( get_el_context(tin, &ctx) )
+  { HistEvent ev;
+    int curr = 0;
+    int rc = FALSE;
+    term_t tail = PL_copy_term_ref(events);
+    term_t head = PL_new_term_ref();
+
+    if ( history(ctx->history, &ev, H_CURR) == 0 )
+      curr = ev.num;
+
+    if ( history(ctx->history, &ev, H_FIRST) == 0 )
+    { if ( !append_ev(tail, head, &ev) )
+	goto out;
+    }
+    while(history(ctx->history, &ev, H_NEXT) == 0)
+    { if ( !append_ev(tail, head, &ev) )
+	goto out;
+    }
+    rc = PL_unify_nil(tail);
+
+  out:
+    history(ctx->history, &ev, H_SET, curr);
+
+    return rc;
+  }
+
+  return FALSE;
+}
+
+
+static int
+get_int_arg(int i, term_t t, int *v)
+{ term_t a;
+
+  if ( (a=PL_new_term_ref()) &&
+       PL_get_arg(i, t, a) &&
+       PL_get_integer_ex(a, v) )
+    return TRUE;
+
+  return FALSE;
+}
+
+static int
+get_bool_arg(int i, term_t t, int *v)
+{ term_t a;
+
+  if ( (a=PL_new_term_ref()) &&
+       PL_get_arg(i, t, a) &&
+       PL_get_bool_ex(a, v) )
+    return TRUE;
+
+  return FALSE;
+}
+
+
+
+static foreign_t
+pl_history(term_t tin, term_t option)
+{ el_context *ctx;
+
+  if ( get_el_context(tin, &ctx) )
+  { atom_t name;
+    size_t arity;
+    int rc = 0;
+
+    if ( PL_get_name_arity(option, &name, &arity) )
+    { HistEvent ev;
+
+      if ( name == ATOM_setsize )
+      { int s;
+	if ( arity != 1 ) goto err_domain;
+	if ( !get_int_arg(1, option, &s) ) return FALSE;
+	rc = history(ctx->history, &ev, H_SETSIZE);
+      } else if ( name == ATOM_clear )
+      { if ( arity != 0 ) goto err_domain;
+
+	rc = history(ctx->history, &ev, H_CLEAR);
+      } else if ( name == ATOM_setunique )
+      { int u;
+	if ( arity != 1 ) goto err_domain;
+	if ( !get_bool_arg(1, option, &u) ) return FALSE;
+	rc = history(ctx->history, &ev, H_SETUNIQUE, u);
+      } else
+      { err_domain:
+	return PL_domain_error("history_action", option);
+      }
+
+      if ( rc == 0 )
+	return TRUE;
+
+      return FALSE;				/* What exception? */
+    }
+
+    return PL_type_error("callable", option);
+  }
+
+  return FALSE;
+}
+
+
+
 		 /*******************************
 		 *	   REGISTRATION		*
 		 *******************************/
@@ -1284,9 +1517,13 @@ install_libedit4pl(void)
   MKATOM(redisplay);
   MKATOM(error);
   MKATOM(fatal);
+  MKATOM(clear);
+  MKATOM(setsize);
+  MKATOM(setunique);
 
   MKFUNCTOR(line, 2);
   MKFUNCTOR(electric, 3);
+  FUNCTOR_pair2 = PL_new_functor(PL_new_atom("-"), 2);
 
   PL_register_foreign("el_wrap",	  4, pl_wrap,	       0);
   PL_register_foreign("el_wrapped",	  1, pl_is_wrapped,    0);
@@ -1301,4 +1538,6 @@ install_libedit4pl(void)
   PL_register_foreign("el_add_history",	  2, pl_add_history,   0);
   PL_register_foreign("el_write_history", 2, pl_write_history, 0);
   PL_register_foreign("el_read_history",  2, pl_read_history,  0);
+  PL_register_foreign("el_history_events",2, pl_history_events,0);
+  PL_register_foreign("el_history",       2, pl_history,       0);
 }
