@@ -44,6 +44,7 @@
 :- use_module(library(pce_report)).
 :- use_module(library(tabular)).
 :- use_module(library(prolog_predicate)).
+:- use_module(library(prolog_code)).
 
 /** <module> GUI frontend for the profiler
 
@@ -288,7 +289,7 @@ variable(data,         prolog, get, "Predicate data").
 initialise(DI, Node:prolog, SortBy:name, F:prof_frame) :->
     "Create from predicate head"::
     send(DI, slot, data, Node),
-    predicate_label(Node.predicate, Key),
+    pce_predicate_label(Node.predicate, Key),
     send_super(DI, initialise, Key),
     send(DI, update_label, SortBy, F).
 
@@ -302,8 +303,19 @@ has_predicate(DI, Test:prolog) :->
     same_pred(Test, Data.predicate).
 
 same_pred(X, X) :- !.
-same_pred(user:Pred, Pred) :- !.
-same_pred(Pred, user:Pred).
+same_pred(QP1, QP2) :-
+    unqualify(QP1, P1),
+    unqualify(QP2, P2),
+    same_pred_(P1, P2).
+
+unqualify(user:X, X) :- !.
+unqualify(X, X).
+
+same_pred_(X, X) :- !.
+same_pred_(Head, Name/Arity) :-
+    pi_head(Name/Arity, Head).
+same_pred_(Head, user:Name/Arity) :-
+    pi_head(Name/Arity, Head).
 
 compare(DI, DI2:prof_dict_item,
         SortBy:name, Order:{normal,reverse},
@@ -577,8 +589,8 @@ show_relative(W, Caller:prolog, Role:name) :->
 :- pce_begin_class(prof_node_text, text,
                    "Show executable object").
 
-variable(context,   any,                   get, "Represented executable").
-variable(role,      {parent,self,child}, get,   "Represented role").
+variable(context,   any,                 get, "Represented executable").
+variable(role,      {parent,self,child}, get, "Represented role").
 
 initialise(T, Context:any, Role:{parent,self,child}, Cycle:[int]) :->
     send(T, slot, context, Context),
@@ -671,10 +683,10 @@ details(T) :->
 
 value(name, Data, Name) :-
     !,
-    predicate_name(Data.predicate, Name).
+    predicate_sort_key(Data.predicate, Name).
 value(label, Data, Label) :-
     !,
-    predicate_label(Data.predicate, Label).
+    pce_predicate_label(Data.predicate, Label).
 value(ticks, Data, Ticks) :-
     !,
     Ticks is Data.ticks_self + Data.ticks_siblings.
@@ -689,40 +701,14 @@ sort_by(flat_profile_by_number_of_redos,     redo,           reverse).
 sort_by(flat_profile_by_name,                name,           normal).
 
 
-%!  predicate_label(+Head, -Label)
+%!  pce_predicate_label(+PI, -Label)
 %
 %   Label is the human-readable identification   for Head. Calls the
 %   hook user:prolog_predicate_name/2.
 
-:- multifile
-    user:prolog_predicate_name/2.
-
-predicate_label(Obj, Label) :-
+pce_predicate_label(Obj, Label) :-
     object(Obj),
     !,
     get(Obj, print_name, Label).
-predicate_label(Head, Label) :-
-    user:prolog_predicate_name(Head, Label),
-    !.
-predicate_label(M:H, Label) :-
-    !,
-    functor(H, Name, Arity),
-    (   hidden_module(M, H)
-    ->  atomic_list_concat([Name, /, Arity], Label)
-    ;   atomic_list_concat([M, :, Name, /, Arity], Label)
-    ).
-predicate_label(H, Label) :-
-    !,
-    functor(H, Name, Arity),
-    atomic_list_concat([Name, /, Arity], Label).
-
-hidden_module(system, _).
-hidden_module(user, _).
-hidden_module(M, H) :-
-    predicate_property(system:H, imported_from(M)).
-
-predicate_name(_:H, Name) :-
-    !,
-    predicate_name(H, Name).
-predicate_name(H, Name) :-
-    functor(H, Name, _Arity).
+pce_predicate_label(PI, Label) :-
+    predicate_label(PI, Label).
