@@ -33,10 +33,6 @@
     POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifdef _WIN64
-#define _WIN32_WINNT 0x0501            /* get RtlCaptureContext() */
-#endif
-
 #define _GNU_SOURCE
 #include "pl-incl.h"
 #include "os/pl-cstack.h"
@@ -209,9 +205,7 @@ print_backtrace(int last)		/* 1..SAVE_TRACES */
 
 void
 bstore_print_backtrace_named(btrace *bt, const char *why)
-{ int done = 0;
-
-  if ( bt )
+{ if ( bt )
   { int me = bt->current-1;
 
     for(;;)
@@ -219,15 +213,12 @@ bstore_print_backtrace_named(btrace *bt, const char *why)
 	me += SAVE_TRACES;
       if ( bt->dumps[me].name && strcmp(bt->dumps[me].name, why) == 0 )
       { print_trace(bt, me);
-	done++;
+	return;
       }
       if ( --me == bt->current-1 )
 	break;
     }
   }
-
-  if ( !done )
-    Sdprintf("No backtrace named %s\n", why);
 }
 
 #endif /*HAVE_LIBUNWIND*/
@@ -443,9 +434,7 @@ print_backtrace(int last)		/* 1..SAVE_TRACES */
 
 void
 bstore_print_backtrace_named(btrace *bt, const char *why)
-{ int done = 0;
-
-  if ( bt )
+{ if ( bt )
   { int me = bt->current-1;
 
     for(;;)
@@ -453,15 +442,12 @@ bstore_print_backtrace_named(btrace *bt, const char *why)
 	me += SAVE_TRACES;
       if ( bt->why[me] && strcmp(bt->why[me], why) == 0 )
       { print_trace(bt, me);
-	done++;
+	return;
       }
       if ( --me == bt->current-1 )
 	break;
     }
   }
-
-  if ( !done )
-    Sdprintf("No backtrace named %s\n", why);
 }
 
 
@@ -826,9 +812,7 @@ print_backtrace(int last)		/* 1..SAVE_TRACES */
 
 void
 bstore_print_backtrace_named(btrace *bt, const char *why)
-{ int done = 0;
-
-  if ( bt )
+{ if ( bt )
   { int me = bt->current-1;
 
     for(;;)
@@ -836,15 +820,12 @@ bstore_print_backtrace_named(btrace *bt, const char *why)
 	me += SAVE_TRACES;
       if ( bt->dumps[me].name && strcmp(bt->dumps[me].name, why) == 0 )
       { print_trace(bt, me);
-	done++;
+	return;
       }
       if ( --me == bt->current-1 )
 	break;
     }
   }
-
-  if ( !done )
-    Sdprintf("No backtrace named %s\n", why);
 }
 
 static LONG WINAPI crashHandler(PEXCEPTION_POINTERS pExceptionInfo)
@@ -1017,6 +998,7 @@ initBackTrace(void)
 {
 }
 
+#ifdef HAVE_SIGNAL
 void
 sigCrashHandler(int sig)
 { int tid;
@@ -1045,4 +1027,13 @@ sigCrashHandler(int sig)
 #endif
 }
 
+#else
+
+void
+sigCrashHandler(int sig)
+{ fatalError("Something went wrong");
+}
+
+#endif /*HAVE_SIGNAL*/
+
 #endif /*BTRACE_DONE*/
