@@ -33,6 +33,10 @@
     POSSIBILITY OF SUCH DAMAGE.
 */
 
+#ifdef __WINDOWS__
+#include <mbstring.h>
+#endif
+
 #include "pl-incl.h"
 #include "pl-ctype.h"
 
@@ -174,7 +178,7 @@ compile_pattern(compiled_pattern *Out, char *p, int curl)
 	  map = &Out->code[Out->size];
 	  Out->size += 16;
 	  if ( Out->size >= MAXCODE )
-	  { warning("Pattern too intptr_t");
+	  { warning("Pattern too long");
 	    return (char *) NULL;
 	  }
 
