@@ -193,6 +193,9 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_TABLING_IDG),
   DEBUG_TOPIC(MSG_TABLING_IDG_CHANGED),
   DEBUG_TOPIC(MSG_TABLING_IDG_REEVAL),
+  DEBUG_TOPIC(MSG_TABLING_EXCEPTION),
+  DEBUG_TOPIC(MSG_TABLING_SHARED),
+  DEBUG_TOPIC(MSG_TABLING_ABOLISH),
 
   DEBUG_TOPIC(TABLING_NO_EARLY_COMPLETION),
   DEBUG_TOPIC(TABLING_NO_SIMPLIFY),
