@@ -240,7 +240,16 @@ freeLingeringDefinitions(ListCell c)
 
 static void
 unallocModule(Module m)
-{ if ( m->public )     destroyHTable(m->public);
+{ GET_LD
+
+#ifdef O_PLMT
+  if ( LD )
+#endif
+  { if ( LD->modules.source == m ) LD->modules.source = MODULE_user;
+    if ( LD->modules.typein == m ) LD->modules.typein = MODULE_user;
+  }
+
+  if ( m->public )     destroyHTable(m->public);
   if ( m->procedures ) destroyHTable(m->procedures);
   if ( m->operators )  destroyHTable(m->operators);
   if ( m->supers )     unallocList(m->supers);
@@ -299,8 +308,7 @@ unlinkSourceFilesModule(Module m)
 
 static int
 destroyModule(Module m)
-{ GET_LD
-  if ( !(m->class != ATOM_temporary ||
+{ if ( !(m->class != ATOM_temporary ||
 	 m->references > 0) )
     Sdprintf("Module %s: class %s; refs %d\n",
 	     PL_atom_chars(m->name), PL_atom_chars(m->class), m->references);
@@ -308,7 +316,11 @@ destroyModule(Module m)
   PL_LOCK(L_MODULE);
   if ( deleteHTable(GD->tables.modules, (void*)m->name) == m )
     set(m, M_DESTROYED);
-  assert(!lookupHTable(GD->tables.modules, (void*)m->name));
+#ifndef NDEBUG
+  { GET_LD
+    assert(!lookupHTable(GD->tables.modules, (void*)m->name));
+  }
+#endif
   PL_UNLOCK(L_MODULE);
 
   releaseModule(m);
@@ -1602,6 +1614,7 @@ retry:
     if ( old != nproc )
     { int shared = unshareDefinition(proc->definition);
       assert(shared > 0);
+      (void)shared;
       freeHeap(nproc, sizeof(*nproc));
       goto retry;
     }
